# coding: utf-8
"""Module with connection utilities."""

import requests
import urllib.parse
from dateutil.parser import parse
from .exceptions import InvalidDateTimeFormat


class BaseRequest:
    """Contains common actions to library."""

    ROOT_API_URL = 'https://api.github.com'

    def get_request_limit(self, access_token):
        """Request Github remaining requests without spend the amount remaining.

        Args:
            access_token: The target access_token.
        Returns:
            int: The amount of remaining requests for a given access_token.

        """
        url = "{0}/rate_limit?access_token={1}"
        response = requests.get(url.format(self.ROOT_API_URL, access_token))
        data = response.json()
        return data['resources']['core'].get("remaining")

    def get_last_modified_header(self, datetime):
        """Return a header with If-Modified-Since attribute.

        Args:
            datetime: The string datetime to be converted.

        Returns:
            dict: The personalized header.

        """
        new_date_format = self._convert_to_rfc1123(datetime)

        return {'If-Modified-Since': new_date_format}

    def _convert_to_rfc1123(self, datetime):
        """Convert an datetime string to RFC1123 format.

        Example
            Input: 2017-10-13T03:03:57Z
            Output: Fri, 13 Oct 2017 03:03:57 GMT

        Args:
            datetime: The string datetime to be converted.
        Returns:
            str: Datetime in RFC1123 format.

        """
        try:
            new_date_format = parse('2017-10-13T03:03:57Z').strftime(
                '%a, %d %b %Y %H:%M:%S GMT')
        except Exception as ex:
            raise InvalidDateTimeFormat({'datetime': datetime})

        return new_date_format

    def get_token(self, access_token):
        """Choose an access_token to be used for each request.

        Args:
            access_token: The priority access_token to be used.
        Returns:
            str: The access_token if exists or None.

        """
        if access_token:
            return access_token
        elif self.default_access_token:
            return self.default_access_token
        else:
            return ''

    def encode_parameters(self, text):
        """Encode special characters to URL pattern.

        Args:
            text: The text to be encoded.

        Returns:
            str: The encoded text.

        """
        return urllib.parse.quote_plus(text)



# coding: utf-8
"""Module that contains all user repository Data Scraping logic."""

import requests
from .utils import BaseRequest
from .exceptions import (InvalidTokenError, RepositoryNameNotFoundError,
                         ApiError, RepositoryIdNotFoundError, ApiRateLimitError)


class RepositoryApi(BaseRequest):
    """Class that has Repository Data Scraping actions."""

    def __init__(self, default_access_token=None):
        """Constructor.

        Args:
            default_access_token: The default GitHub access_token

        If you don't provide an access_token, your number of requests will be limited to 60 requests per hour, acording with GitHub REST API v3.
        """
        self.default_access_token = default_access_token

    def repository_by_id(self, repository_id, access_token=None):
        """Return a repository with given repository ID."""
        url = "{0}/repositories/{1}{2}"
        access_token = self.get_token(access_token)
        token_arg = ''

        if access_token != '':
            token_arg = "?access_token={}".format(access_token)

        response = requests.get(
            url.format(self.ROOT_API_URL, repository_id, token_arg))

        self._check_common_status_code(response, access_token)

        if response.status_code == requests.codes.not_found:
            raise RepositoryIdNotFoundError({'repository_id': repository_id})

        return response.json()

    def repository_by_name(self, username, repository_name, access_token=None):
        """Return a repository with given repository_name and username."""
        url = "{0}/repos/{1}/{2}{3}"
        access_token = self.get_token(access_token)
        token_arg = ''

        if access_token != '':
            token_arg = "?access_token={}".format(access_token)

        response = requests.get(
            url.format(
                self.ROOT_API_URL, username, repository_name, token_arg)
        )

        self._check_common_status_code(response, access_token)

        if response.status_code == requests.codes.not_found:
            raise RepositoryNameNotFoundError(
                {'repository_name': repository_name, 'username': username})

        return response.json()

    def get_all_data(self, repository_id=None, repository_name=None, access_token=None):
        """Request all repository data from a given repository ID or name."""
        data = {}

        if repository_id:
            root_data = self.repository_by_id(repository_id, access_token)
            data['id'] = root_data['id']
            data['name'] = root_data['name']
            data['private'] = root_data['private']
            data['description'] = root_data['description']
            data['fork'] = root_data['fork']
            data['id'] = root_data['id']
            data['created_at'] = root_data['created_at']
            data['updated_at'] = root_data['updated_at']
            data['pushed_at'] = root_data['pushed_at']
            data['homepage'] = root_data['homepage']
            data['size'] = root_data['size']
            data['stargazers_count'] = root_data['stargazers_count']
            data['watchers_count'] = root_data['watchers_count']
            data['language'] = root_data['language']
            data['has_issues'] = root_data['has_issues']
            data['has_projects'] = root_data['has_projects']
            data['has_downloads'] = root_data['has_downloads']
            data['has_wiki'] = root_data['has_wiki']
            data['has_pages'] = root_data['has_pages']
            data['forks_count'] = root_data['forks_count']
            data['mirror_url'] = root_data['mirror_url']
            data['archived'] = root_data['archived']
            data['open_issues_count'] = root_data['open_issues_count']
            data['forks'] = root_data['forks']
            data['open_issues'] = root_data['open_issues']
            data['watchers'] = root_data['watchers']
            data['default_branch'] = root_data['default_branch']
            data['network_count'] = root_data['network_count']
            data['subscribers_count'] = root_data['subscribers_count']

            data['branches'] = self.get_branches_by_id(
                repository_id, access_token)
            data['comments'] = self.get_comments_by_id(
                repository_id, access_token)
            data['commits'] = self.get_commits_by_id(
                repository_id, access_token)
            data['contents'] = self.get_contents_by_id(
                repository_id, access_token)
            data['contributors'] = self.get_contributors_by_id(
                repository_id, access_token)
            data['events'] = self.get_events_by_id(
                repository_id, access_token)
            data['issues'] = self.get_issues_by_id(
                repository_id, access_token)
            data['labels'] = self.get_labels_by_id(
                repository_id, access_token)
            data['languages'] = self.get_languages_by_id(
                repository_id, access_token)
            data['pulls'] = self.get_pulls_by_id(
                repository_id, access_token)
            data['subscribers'] = self.get_subscribers_by_id(
                repository_id, access_token)
            data['tags'] = self.get_tags_by_id(
                repository_id, access_token)

        return data

    def commits_by_name(self, username, repository_name, access_token):
        """Return repository commits from a given username.

        Arguments:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "commits", access_token)

    def contributors_by_name(self, username, repository_name, access_token):
        """Return repository contributors from a given username.

        Arguments:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "contributors", access_token)

    def issues_by_name(self, username, repository_name, access_token):
        """Return repository issues from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "issues", access_token)

    def events_by_name(self, username, repository_name, access_token):
        """Return repository events from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "events", access_token)

    def branches_by_name(self, username, repository_name, access_token):
        """Return repository branches from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "branches", access_token)

    def tags_by_name(self, username, repository_name, access_token):
        """Return repository tags from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "tags", access_token)

    def languages_by_name(self, username, repository_name, access_token):
        """Return repository languages from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "languages", access_token)

    def subscribers_by_name(self, username, repository_name, access_token):
        """Return repository subscribers from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "subscribers", access_token)

    def comments_by_name(self, username, repository_name, access_token):
        """Return repository comments from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "comments", access_token)

    def contents_by_name(self, username, repository_name, access_token):
        """Return repository contents from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "contents", access_token)

    def pulls_by_name(self, username, repository_name, access_token):
        """Return repository pulls from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "pulls", access_token)

    def labels_by_name(self, username, repository_name, access_token):
        """Return repository labels from a given username.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_name(
            username, repository_name, "labels", access_token)

    def commits_by_id(self, repository_id, access_token=None):
        """Return repository commits from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "commits", access_token)

    def contributors_by_id(self, repository_id, access_token=None):
        """Return repository contributors from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "contributors", access_token)

    def issues_by_id(self, repository_id, access_token=None):
        """Return repository issues from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "issues", access_token)

    def events_by_id(self, repository_id, access_token=None):
        """Return repository events from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "events", access_token)

    def branches_by_id(self, repository_id, access_token=None):
        """Return repository branches from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "branches", access_token)

    def tags_by_id(self, repository_id, access_token=None):
        """Return repository tags from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(repository_id, "tags", access_token)

    def languages_by_id(self, repository_id, access_token=None):
        """Return languages tags from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "languages", access_token)

    def subscribers_by_id(self, repository_id, access_token=None):
        """Return subscribers tags from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "subscribers", access_token)

    def comments_by_id(self, repository_id, access_token=None):
        """Return comments tags from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "comments", access_token)

    def contents_by_id(self, repository_id, access_token=None):
        """Return contents tags from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "contents", access_token)

    def pulls_by_id(self, repository_id, access_token=None):
        """Return pulls tags from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "pulls", access_token)

    def labels_by_id(self, repository_id, access_token=None):
        """Return labels tags from a given username and repository ID.

        Args:
            repository_id: An existent user's repository ID.
            access_token: GitHub OAuth2 access token.
        """
        return self._complete_request_by_id(
            repository_id, "labels", access_token)

    def _complete_request_by_name(self, username, repository_name, complement, access_token):
        """Complements a repository data request by name.

        Args:
            username: Github username.
            repository_name: An existent user's repository name.
            access_token: GitHub OAuth2 access token.
        """
        url = "{0}/repos/{1}/{2}/{3}{4}"
        access_token = self.get_token(access_token)
        token_arg = ''

        if access_token != '':
            token_arg = "?access_token={}".format(access_token)

        response = requests.get(
            url.format(
                self.ROOT_API_URL, username, repository_name, complement,
                token_arg
            )
        )

        self._check_common_status_code(response, access_token)

        if response.status_code == requests.codes.not_found:
            raise RepositoryNameNotFoundError(
                {'repository_name': repository_name, 'username': username})

        return response.json()

    def _complete_request_by_id(self, repository_id, complement, access_token):
        """Complements a repository data request by ID.

        Args:
            repository_id: An existent user's repository ID.
            complement: A resource to be requested.
            access_token: GitHub OAuth2 access token.
        """
        url = "{0}/repositories/{1}/{2}{3}"
        access_token = self.get_token(access_token)
        token_arg = ''

        if access_token != '':
            token_arg = "?access_token={}".format(access_token)

        response = requests.get(
            url.format(self.ROOT_API_URL, repository_id, complement, token_arg))

        self._check_common_status_code(response, access_token)

        if response.status_code == requests.codes.not_found:
            raise RepositoryIdNotFoundError({'repository_id': repository_id})

        return response.json()

    def _check_common_status_code(self, response, access_token):
        remaining = int(response.headers['X-RateLimit-Remaining'])

        if response.status_code == requests.codes.forbidden and remaining == 0:
            raise ApiRateLimitError(
                {'X-RateLimit-Remaining': remaining,
                 'X-RateLimit-Limit': response.headers['X-RateLimit-Limit']})
        elif response.status_code == requests.codes.unauthorized:
            raise InvalidTokenError({'access_token': access_token})
        elif response.status_code >= 500 and response.status_code <= 509:
            raise ApiError()



# coding: utf-8
"""Module that contains all personalized exceptions used in this library."""

class BaseError(Exception):
    """Base Exception to be inherited by all Githon exceptions.

    Args:
        kwargs: Dict with attributes that will be logged in message error.
    """

    def __init__(self, kwargs):
        """Initialize with keyword arguments."""
        self.kwargs = kwargs


class ApiError(Exception):
    """Exception raised when GitHub Server returns status code equals to 500."""

    def __str__(self):
        return "An unexpected error occurred on the Github API server"


class InvalidQueryError(Exception):
    """Exception raised by searches without query string."""

    def __str__(self):
        """Return error description."""
        return "Github Query string not provided or is invalid."


class ApiRateLimitError(BaseError):
    """Exception raised by GitHub API rate limit exceeded."""

    def __str__(self):
        """Return error description."""
        return "GitHub API rate limit for your token was reached. X-RateLimit-Limit: {0} - X-RateLimit-Remaining: {1}".format(self.kwargs.get('X-RateLimit-Limit', None), self.kwargs.get('X-RateLimit-Remaining', None))


class UserNotFoundError(BaseError):
    """Exception raised by searches with an unexistent username."""

    def __str__(self):
        """Return error description."""
        return "Github user '{}' not exists.".format(
            self.kwargs.get('user', None))


class RepositoryIdNotFoundError(BaseError):
    """Exception raised by searches with an unexistent repository ID."""

    def __str__(self):
        """Return error description."""
        return "Reposity with ID '{}' does not exists.".format(
            self.kwargs.get('repository_id', None))


class RepositoryNameNotFoundError(BaseError):
    """Exception raised by searches with an unexistent repository name."""

    def __str__(self):
        """Return error description."""
        return "Reposity with name '{0}' not exists for user '{1}'".format(
            self.kwargs.get('repository_id', None),
            self.kwargs.get('username', None))


class InvalidTokenError(BaseError):
    """Exception raised by unexistents or expired access token."""

    def __str__(self):
        """Return error description."""
        return "The access token '{}' doesn't have permission to access the requested resource or has been expired.".format(
            self.kwargs.get('access_token', None))

class InvalidDateTimeFormat(BaseError):
    """Exception raised by invalid datetime converts."""

    def __str__(self):
        """Return error description."""
        return "The datetime '{}' doesn't have a valid format to convertion.".format(
            self.kwargs.get('datetime', None))



# coding: utf-8
"""Module that contains all GitHub data scraping logic."""
import requests
from .utils import BaseRequest
from .exceptions import (InvalidTokenError, UserNotFoundError, ApiError,
                         InvalidQueryError, ApiRateLimitError)


class GithubApi(BaseRequest):
    """Class that controls all Github API v3 requests."""

    def __init__(self, default_access_token=None):
        """Constructor.

        Args:
            access_token: The default GitHub access_token.

        If you don't provide an access_token, your number of requests will be limited to 60 requests per hour, acording with GitHub REST API v3.
        See more in https://developer.github.com/v3/#rate-limiting
        """
        self.default_access_token = default_access_token

    def user_by_id(self, user_id, access_token=None, last_modified_date=None):
        """Get user by User ID.

        Args:
            user_id: The Github profile ID.
            access_token: GitHub OAuth2 access token.
            last_modified_date: Last modified Datetime.

        last_modified_date arg reduces request spends.
        See more in https://developer.github.com/v3/#conditional-requests.

        Returns:
            dict: A dictionary with Github profile data.

        """
        return self._complete_user_request(
            "user", user_id, access_token, last_modified_date)

    def user_by_username(self, username, access_token=None, last_modified_date=None):
        """Get user by username.

        Args:
            username: The Github profile username.
            access_token: GitHub OAuth2 access token.
            last_modified_date: Last modified Datetime.

        last_modified_date arg reduces request spends.
        See more in https://developer.github.com/v3/#conditional-requests.

        Returns:
            dict: A dictionary with Github profile data.

        """
        return self._complete_user_request(
            "users", username, access_token, last_modified_date)

    def user_emails(self, access_token):
        """Retrieve a list of emails from a given access_token.

        Arguments:
            access_token: OAuth2 access token authorized by account owner.

        Returns:
            dict: The profile emails data.

        """
        url = "{0}/user/emails?access_token={1}"
        response = requests.get(url.format(self.ROOT_API_URL, access_token))
        remaining = int(response.headers['X-RateLimit-Remaining'])

        if response.status_code == requests.codes.forbidden and remaining == 0:
            raise ApiRateLimitError(
                {'X-RateLimit-Remaining': remaining,
                 'X-RateLimit-Limit': response.headers['X-RateLimit-Limit']})
        elif response.status_code == requests.codes.unauthorized:
            raise InvalidTokenError({'access_token': access_token})
        elif response.status_code >= 500 and response.status_code <= 509:
            raise ApiError()

        return response.json()

    def followers_by_id(self, user_id, access_token=None):
        """Return followers from a given User ID.

        Args:
            user_id: Github User ID.
            access_token: GitHub OAuth2 access token.

        Returns:
            dict: A dictionary with summary user's followers data.

        """
        return self._complete_resource_request(
            "user", user_id, "followers", access_token)

    def followers_by_username(self, username, access_token=None):
        """Return followers from a given username.

        Args:
            username: Github username.
            access_token: GitHub OAuth2 access token.

        Returns:
            dict: A dictionary with summary user's followers data.

        """
        return self._complete_resource_request(
            "users", username, "followers", access_token)

    def following_by_id(self, user_id, access_token=None):
        """Return following list from a given User ID.

        Args:
            user_id: Github User ID.
            access_token: GitHub OAuth2 access token.

        Returns:
            dict: A dictionary with summary user's following data.

        """
        return self._complete_resource_request(
            "user", user_id, "following", access_token)

    def following_by_username(self, username, access_token=None):
        """Return following list from a given username.

        Args:
            user_id: Github username.
            access_token: GitHub OAuth2 access token.

        Returns:
            dict: A dictionary with summary user's following data.

        """
        return self._complete_resource_request(
            "users", username, "following", access_token)

    def gists_by_id(self, user_id, access_token=None):
        """Return gists from a given User ID.

        Args:
            user_id: Github User ID.
            access_token: GitHub OAuth2 access token.

        Returns:
            dict: A dictionary with summary user's gist data.

        """
        return self._complete_resource_request(
            "user", user_id, "gists", access_token)

    def gists_by_username(self, username, access_token=None):
        """Return gists from a given username.

        Args:
            username: Github username.
            access_token: GitHub OAuth2 access token.

        Returns:
            dict: A dictionary with summary user's gist data.

        """
        return self._complete_resource_request(
            "users", username, "gists", access_token)

    def repositories_by_id(self, user_id, access_token=None):
        """Return repositories from a given User ID.

        Args:
            user_id: Github User ID.
            access_token: GitHub OAuth2 access token.

        Returns:
            dict: A dictionary with summary user's repositories data.

        """
        return self._complete_resource_request(
            "user", user_id, "repos", access_token)

    def repositories_by_username(self, username, access_token=None):
        """Return repositories from a given username.

        Args:
            username: Github username.
            access_token: GitHub OAuth2 access token.

        Returns:
            dict: A dictionary with summary user's repositories data.

        """
        return self._complete_resource_request(
            "users", username, "repos", access_token)

    def _complete_user_request(self, kind, user, access_token, last_modified_date):
        """Complements an user data request from a given User.

        Args:
            kind: 'users' if username passed or 'user' if user id passed.
            user: Github User ID or Username
            access_token: GitHub OAuth2 access token.
            last_modified_date: Last modified Datetime.

        last_modified_date arg reduces request spends.
        See more in https://developer.github.com/v3/#conditional-requests.

        Returns:
            dict: A dictionary with Github profile data.

        """
        url = "{0}/{1}/{2}{3}"
        headers = None
        data = {}
        access_token = self.get_token(access_token)
        token_arg = ''

        if access_token != '':
            token_arg = "?access_token={}".format(access_token)

        if last_modified_date:
            headers = self.get_last_modified_header(last_modified_date)

        response = requests.get(
            url.format(self.ROOT_API_URL, kind, user, token_arg),
            headers=headers)

        self._check_status_code(response, user, access_token)

        if response.text != '':
            data = response.json()

        return data

    def _complete_resource_request(self, kind, user, complement, access_token):
        """Complements an user data request from a given User.

        Args:
            kind: 'users' if username passed or 'user' if user id passed.
            user: Github User ID or Username
            complement: A resource to be requested.
            access_token: GitHub OAuth2 access token.

        Returns:
            dict: A dictionary with requested data.

        """
        url = "{0}/{1}/{2}/{3}{4}"
        access_token = self.get_token(access_token)
        token_arg = ''

        if access_token != '':
            token_arg = "?access_token={}".format(access_token)

        response = requests.get(url.format(
            self.ROOT_API_URL, kind, user, complement, token_arg))

        self._check_status_code(response, user, access_token)

        return response.json()

    def _check_status_code(self, response, user, access_token):
        """Check status codes and raise Exceptions if necessary.

        Args:
            response: HTTP Response object from requests library.
            user: Github UID or Username.
            access_token: GitHub OAuth2 access token.
        """
        remaining = int(response.headers['X-RateLimit-Remaining'])

        if response.status_code == requests.codes.not_found:
            raise UserNotFoundError({'user': user})
        if response.status_code == requests.codes.forbidden and remaining == 0:
            raise ApiRateLimitError(
                {'X-RateLimit-Remaining': remaining,
                 'X-RateLimit-Limit': response.headers['X-RateLimit-Limit']})
        elif response.status_code == requests.codes.unauthorized:
            raise InvalidTokenError({'access_token': access_token})
        elif response.status_code >= 500 and response.status_code <= 509:
            raise ApiError()

    def search_users(self, query, page=1, per_page=100, access_token=None):
        """Retrieve users with a given query.

        Args:
            query: A Github query string.
            page: Controls the pagination.
            per_page: Controls the number os results per page.
            access_token: GitHub OAuth2 access token.
        Returns:
            dict: A list of Github users that matches with query.

        """
        # repos:>=1
        access_token = self.get_token(access_token)
        token_arg = ''

        if access_token != '':
            token_arg = "&access_token={}".format(access_token)

        url = "{0}/search/users?q={1}&page={2}&per_page={3}&type=Users{4}"

        response = requests.get(
            url.format(self.ROOT_API_URL, self.encode_parameters(query), page,
                       per_page, token_arg))

        remaining = int(response.headers['X-RateLimit-Remaining'])

        if response.status_code == requests.codes.forbidden and remaining == 0:
            raise ApiRateLimitError(
                {'X-RateLimit-Remaining': remaining,
                 'X-RateLimit-Limit': response.headers['X-RateLimit-Limit']})
        elif response.status_code == requests.codes.unauthorized:
            raise InvalidTokenError({'access_token': access_token})
        elif response.status_code == requests.codes.not_found:
            raise InvalidQueryError()
        elif response.status_code >= 500 and response.status_code <= 509:
            raise ApiError()

        return response.json()



# coding=utf-8
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

# import githon

requirements = open('requirements.pip').read().splitlines()

kw = dict(
    name = 'githon',
    version = '0.5.0',
    description = 'A simple Data Scraping library for GitHub REST API v3',
    long_description = open('README.md', 'r').read(),
    author = 'Marcos Vinícius Brás',
    author_email = 'marcosvbras@gmail.com',
    url = 'https://github.com/marcosvbras/githon',
    download_url = 'https://github.com/marcosvbras/githon',
    license='Apache',
    py_modules = ["githon"],
    packages = ["githon"],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    keywords='data github scraping api',
    install_requires=requirements,
    python_requires='~=3.3',
)

setup(**kw)
